@@ -1,22 +1,25 @@
+from __future__ import annotations
+
 import collections
+import functools
 import json
 import os
-import re
 import shutil
 import sys
-from contextlib import contextmanager
-from io import BytesIO
+from dataclasses import dataclass
+from io import BytesIO, StringIO
 from pathlib import Path
-from typing import Callable, Dict, Iterable, List, Optional, Tuple
-from urllib.parse import urlparse
+from typing import Callable, Iterable, Mapping
+from urllib.parse import unquote, urlparse
 
 import pytest
-from click.testing import CliRunner
-from pip._internal.vcs import versioncontrol
-from pip._vendor import requests
+import requests
+from packaging.version import parse as parse_version
+from unearth.vcs import Git, vcs_support
 
 from pdm._types import CandidateInfo
-from pdm.cli.actions import do_init, do_use
+from pdm.cli.actions import do_init
+from pdm.cli.hooks import HookManager
 from pdm.core import Core
 from pdm.exceptions import CandidateInfoNotFound
 from pdm.models.candidates import Candidate
@@ -27,37 +30,51 @@ from pdm.models.requirements import (
     filter_requirements_with_extras,
     parse_requirement,
 )
+from pdm.models.session import PDMSession
 from pdm.project.config import Config
-from pdm.utils import get_finder, normalize_name
+from pdm.project.core import Project
+from pdm.utils import find_python_in_path, normalize_name, path_to_url
 from tests import FIXTURES
 
-os.environ["CI"] = "1"
-
-
-@contextmanager
-def temp_environ():
-    environ = os.environ.copy()
-    try:
-        yield
-    finally:
-        os.environ.clear()
-        os.environ.update(environ)
+os.environ.update(CI="1", PDM_CHECK_UPDATE="0")
 
 
 class LocalFileAdapter(requests.adapters.BaseAdapter):
-    def __init__(self, base_path):
+    def __init__(self, aliases, overrides=None, strip_suffix=False):
         super().__init__()
-        self.base_path = base_path
+        self.aliases = sorted(
+            aliases.items(), key=lambda item: len(item[0]), reverse=True
+        )
+        self.overrides = overrides if overrides is not None else {}
+        self.strip_suffix = strip_suffix
         self._opened_files = []
 
+    def get_file_path(self, path):
+        for prefix, base_path in self.aliases:
+            if path.startswith(prefix):
+                file_path = base_path / path[len(prefix) :].lstrip("/")
+                if not self.strip_suffix:
+                    return file_path
+                return next(
+                    (p for p in file_path.parent.iterdir() if p.stem == file_path.name),
+                    None,
+                )
+        return None
+
     def send(
         self, request, stream=False, timeout=None, verify=True, cert=None, proxies=None
     ):
-        file_path = self.base_path / urlparse(request.url).path.lstrip("/")
+        request_path = urlparse(request.url).path
+        file_path = self.get_file_path(request_path)
         response = requests.models.Response()
         response.url = request.url
         response.request = request
-        if not file_path.exists():
+        if request_path in self.overrides:
+            response.status_code = 200
+            response.reason = "OK"
+            response.raw = BytesIO(self.overrides[request_path])
+            response.headers["Content-Type"] = "text/html"
+        elif file_path is None or not file_path.exists():
             response.status_code = 404
             response.reason = "Not Found"
             response.raw = BytesIO(b"Not Found")
@@ -76,21 +93,18 @@ class LocalFileAdapter(requests.adapters.BaseAdapter):
         self._opened_files.clear()
 
 
-class MockVersionControl(versioncontrol.VersionControl):
-    def obtain(self, dest, url, verbosity=0):
-        url, _ = self.get_url_rev_options(url)
-        path = os.path.splitext(os.path.basename(urlparse(str(url)).path))[0]
+class MockGit(Git):
+    def fetch_new(self, location, url, rev, args):
+        path = os.path.splitext(os.path.basename(unquote(urlparse(str(url)).path)))[0]
         mocked_path = FIXTURES / "projects" / path
-        shutil.copytree(mocked_path, dest)
+        shutil.copytree(mocked_path, location)
 
-    @classmethod
-    def get_revision(cls, location):
+    def get_revision(self, location: Path) -> str:
         return "1234567890abcdef"
 
-    def is_immutable_rev_checkout(self, url: str, dest: str) -> bool:
-        if "@1234567890abcdef" in url:
-            return True
-        return super().is_immutable_rev_checkout(url, dest)
+    def is_immutable_revision(self, location, link) -> bool:
+        rev = self.get_url_and_rev_options(link)[1]
+        return rev == "1234567890abcdef"
 
 
 class _FakeLink:
@@ -115,7 +129,7 @@ class TestRepository(BaseRepository):
 
     def _get_dependencies_from_fixture(
         self, candidate: Candidate
-    ) -> Tuple[List[str], str, str]:
+    ) -> tuple[list[str], str, str]:
         try:
             pypi_data = self._pypi_data[candidate.req.key][candidate.version]
         except KeyError:
@@ -133,11 +147,15 @@ class TestRepository(BaseRepository):
             self._get_dependencies_from_metadata,
         )
 
-    def get_hashes(self, candidate: Candidate) -> Optional[Dict[str, str]]:
+    def get_hashes(self, candidate: Candidate) -> dict[str, str] | None:
         return {}
 
     def _find_candidates(self, requirement: Requirement) -> Iterable[Candidate]:
-        for version, candidate in self._pypi_data.get(requirement.key, {}).items():
+        for version, candidate in sorted(
+            self._pypi_data.get(requirement.key, {}).items(),
+            key=lambda item: parse_version(item[0]),
+            reverse=True,
+        ):
             c = Candidate(
                 requirement,
                 name=requirement.project_name,
@@ -220,18 +238,14 @@ def working_set(mocker, repository):
     yield rv
 
 
-def get_local_finder(*args, **kwargs):
-    finder = get_finder(*args, **kwargs)
-    finder.session.mount("http://fixtures.test/", LocalFileAdapter(FIXTURES))
-    return finder
-
-
-@pytest.fixture(autouse=True)
-def pip_global_tempdir_manager():
-    from pdm.models.pip_shims import global_tempdir_manager
-
-    with global_tempdir_manager():
-        yield
+def get_pypi_session(*args, overrides=None, **kwargs):
+    session = PDMSession(*args, **kwargs)
+    session.mount("http://fixtures.test/", LocalFileAdapter({"/": FIXTURES}))
+    session.mount(
+        "https://my.pypi.org/",
+        LocalFileAdapter({"/simple": FIXTURES / "index"}, overrides, strip_suffix=True),
+    )
+    return session
 
 
 def remove_pep582_path_from_pythonpath(pythonpath):
@@ -244,6 +258,8 @@ def remove_pep582_path_from_pythonpath(pythonpath):
 @pytest.fixture()
 def core():
     old_config_map = Config._config_map.copy()
+    # Turn off use_venv by default, for testing
+    Config._config_map["python.use_venv"].default = False
     main = Core()
     yield main
     # Restore the config items
@@ -251,7 +267,12 @@ def core():
 
 
 @pytest.fixture()
-def project_no_init(tmp_path, mocker, core):
+def index():
+    return {}
+
+
+@pytest.fixture()
+def project_no_init(tmp_path, mocker, core, index, monkeypatch):
     test_home = tmp_path / ".pdm-home"
     test_home.mkdir(parents=True)
     test_home.joinpath("config.toml").write_text(
@@ -262,33 +283,45 @@ def project_no_init(tmp_path, mocker, core):
     p = core.create_project(
         tmp_path, global_config=test_home.joinpath("config.toml").as_posix()
     )
-    mocker.patch("pdm.utils.get_finder", get_local_finder)
-    mocker.patch("pdm.models.environment.get_finder", get_local_finder)
+    p.global_config["venv.location"] = str(tmp_path / "venvs")
+    mocker.patch(
+        "pdm.models.environment.PDMSession",
+        functools.partial(get_pypi_session, overrides=index),
+    )
     tmp_path.joinpath("caches").mkdir(parents=True)
     p.global_config["cache_dir"] = tmp_path.joinpath("caches").as_posix()
-    do_use(p, getattr(sys, "_base_executable", sys.executable))
-    with temp_environ():
-        os.environ.pop("VIRTUAL_ENV", None)
-        os.environ.pop("CONDA_PREFIX", None)
-        os.environ.pop("PEP582_PACKAGES", None)
-        os.environ.pop("NO_SITE_PACKAGES", None)
-        pythonpath = os.environ.pop("PYTHONPATH", "")
-        pythonpath = remove_pep582_path_from_pythonpath(pythonpath)
-        if pythonpath:
-            os.environ["PYTHONPATH"] = pythonpath
-        yield p
+    p.project_config["python.path"] = find_python_in_path(sys.base_prefix).as_posix()
+    monkeypatch.delenv("VIRTUAL_ENV", raising=False)
+    monkeypatch.delenv("CONDA_PREFIX", raising=False)
+    monkeypatch.delenv("PEP582_PACKAGES", raising=False)
+    monkeypatch.delenv("NO_SITE_PACKAGES", raising=False)
+    pythonpath = os.getenv("PYTHONPATH", "")
+    pythonpath = remove_pep582_path_from_pythonpath(pythonpath)
+    if pythonpath:
+        monkeypatch.setenv("PYTHONPATH", pythonpath)
+    yield p
 
 
 @pytest.fixture()
 def local_finder(project_no_init, mocker):
-    return_value = ["--no-index", "--find-links", str(FIXTURES / "artifacts")]
-    mocker.patch("pdm.utils.prepare_pip_source_args", return_value=return_value)
+    artifacts_dir = str(FIXTURES / "artifacts")
+    return_value = ["--no-index", "--find-links", artifacts_dir]
     mocker.patch("pdm.builders.base.prepare_pip_source_args", return_value=return_value)
+    project_no_init.tool_settings["source"] = [
+        {
+            "type": "find_links",
+            "verify_ssl": False,
+            "url": path_to_url(artifacts_dir),
+            "name": "pypi",
+        }
+    ]
+    project_no_init.write_pyproject()
 
 
 @pytest.fixture()
 def project(project_no_init):
-    do_init(project_no_init, "test_project", "0.0.0")
+    hooks = HookManager(project_no_init, ["post_init"])
+    do_init(project_no_init, "test_project", "0.0.0", hooks=hooks)
     # Clean the cached property
     project_no_init._environment = None
     return project_no_init
@@ -325,16 +358,9 @@ def repository(project, mocker, local_finder):
 
 
 @pytest.fixture()
-def vcs(mocker):
-    ret = MockVersionControl()
-    mocker.patch(
-        "pip._internal.vcs.versioncontrol.VcsSupport.get_backend", return_value=ret
-    )
-    mocker.patch(
-        "pip._internal.vcs.versioncontrol.VcsSupport.get_backend_for_scheme",
-        return_value=ret,
-    )
-    yield ret
+def vcs(monkeypatch):
+    monkeypatch.setattr(vcs_support, "_registry", {"git": MockGit})
+    return
 
 
 @pytest.fixture(params=[False, True])
@@ -347,14 +373,61 @@ def is_dev(request):
     return request.param
 
 
+@dataclass
+class RunResult:
+    exit_code: int
+    stdout: str
+    stderr: str
+    exception: Exception | None = None
+
+    @property
+    def output(self) -> str:
+        return self.stdout
+
+    @property
+    def outputs(self) -> str:
+        return self.stdout + self.stderr
+
+    def print(self):
+        print("# exit code:", self.exit_code)
+        print("# stdout:", self.stdout, sep="\n")
+        print("# stderr:", self.stderr, sep="\n")
+
+
 @pytest.fixture()
-def invoke(core):
-    runner = CliRunner(mix_stderr=False)
+def invoke(core, monkeypatch):
+    def caller(
+        args,
+        strict: bool = False,
+        input: str | None = None,
+        obj: Project | None = None,
+        env: Mapping[str, str] | None = None,
+        **kwargs,
+    ):
+        __tracebackhide__ = True
+
+        stdin = StringIO(input)
+        stdout = StringIO()
+        stderr = StringIO()
+        exit_code = 0
+        exception = None
+
+        with monkeypatch.context() as m:
+            m.setattr("sys.stdin", stdin)
+            m.setattr("sys.stdout", stdout)
+            m.setattr("sys.stderr", stderr)
+            for key, value in (env or {}).items():
+                m.setenv(key, value)
+            try:
+                core.main(args, "pdm", obj=obj, **kwargs)
+            except SystemExit as e:
+                exit_code = e.code
+            except Exception as e:
+                exit_code = 1
+                exception = e
+
+        result = RunResult(exit_code, stdout.getvalue(), stderr.getvalue(), exception)
 
-    def caller(args, strict=False, **kwargs):
-        result = runner.invoke(
-            core, args, catch_exceptions=not strict, prog_name="pdm", **kwargs
-        )
         if strict and result.exit_code != 0:
             raise RuntimeError(
                 f"Call command {args} failed({result.exit_code}): {result.stderr}"
@@ -364,24 +437,11 @@ def invoke(core):
     return caller
 
 
-@pytest.fixture()
-def index():
-    from pip._internal.index.collector import HTMLPage, LinkCollector
+BACKENDS = ["virtualenv", "venv"]
 
-    old_fetcher = LinkCollector.fetch_page
-    fake_index = {}
 
-    def fetch_page(self, location):
-        m = re.search(r"/simple/([^/]+)/?", location.url)
-        if not m:
-            return old_fetcher(self, location)
-        name = m.group(1)
-        if name not in fake_index:
-            fake_index[name] = (FIXTURES / f"index/{name}.html").read_bytes()
-        return HTMLPage(
-            fake_index[name], "utf-8", location.url, cache_link_parsing=False
-        )
-
-    LinkCollector.fetch_page = fetch_page
-    yield fake_index
-    LinkCollector.fetch_page = old_fetcher
+@pytest.fixture(params=BACKENDS)
+def venv_backends(project, request):
+    project.project_config["venv.backend"] = request.param
+    project.project_config["python.use_venv"] = True
+    shutil.rmtree(project.root / "__pypackages__", ignore_errors=True)
